# This file was generated by Rcpp::compileAttributes
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' icd
#' Changes the current working directory in iRODS or home if empty
#' @param rods_path The iRODS destination path to cd into
#'
icd <- function(rods_path = "", verbose = FALSE) {
    .Call('rirods_icd', PACKAGE = 'rirods', rods_path, verbose)
}

#' icp
#' Copies files on iRODS server. This implementation does not allow to user physical path nor Resource server option.
#' @param src_path Local source file path
#' @param dest_path iRODS destination file path
#' @param force write data-object even it exists already; overwrite it
#' @param calculate_checksum calculate a checksum on the data server-side, and store it in the catalog
#' @param checksum verify checksum - calculate and verify the checksum on the data,
#'                 both client-side and server-side, and store it in the catalog
#' @param progress output the progress of the download
#' @param verbose verbose
#'
icp <- function(src_path, dest_path, force = FALSE, calculate_checksum = FALSE, checksum = FALSE, progress = FALSE, verbose = FALSE) {
    .Call('rirods_icp', PACKAGE = 'rirods', src_path, dest_path, force, calculate_checksum, checksum, progress, verbose)
}

#' r_iexit
#' Performs the iRODS session logout
#' @param full Optional parameter 'full' will terminate the iRODS session, default "FALSE"
#'
iexit <- function(full = FALSE) {
    invisible(.Call('rirods_iexit', PACKAGE = 'rirods', full))
}

#' r_iinit
#' Performs the iRODS session login and obtains the session token
#' Unlike real 'iinit' command it does not perform user environment actions
#' @param p Secret password to be passed from R session
#'
iinit <- function(pass) {
    .Call('rirods_iinit', PACKAGE = 'rirods', pass)
}

#' imeta_list
#' Data frame veriant for inspecting iRODS metadata
#' @param command         add, ls, rm, cp, qu
#' @param object_type     iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param dst_object_type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param dst_name        iRODS object name(cp only)
#' @param avu             format "Attribute;Value;Unit"
#' @param new_avu         format "Attribute;Value;Unit"
#' @param query           See, icommand help 'imeta help qu' for details, help is printed if empty
#'
NULL

#' imeta_qu
#' Query iRODS objects (files, collections) using their AVUs
#'
#' imeta_qu( type="d", query="distance '<=' 12")
#'
#' When querying dataObjects (d) or collections (C) additional conditions
#' (AttName Op AttVal) may be given separated by 'and', for example:
#'
#' imeta_qu( type="d", query="a = b and c '<' 10")
#'
#' Or a single 'or' can be given for the same AttName, for example
#'
#' imeta_qu( type="d", query="r '<' 5 or '>' 7")
#'
#' You can also query in numeric mode (instead of as strings) by adding 'n'
#' in front of the test condition, for example:
#'
#' imeta_qu( type="d", query="r 'n<' 123")
#'
#' which causes it to cast the AVU column to numeric (decimal) in the SQL.
#' In numeric mode, if any of the named AVU values are non-numeric, a SQL
#' error will occur but this avoids problems when comparing numeric strings
#' of different lengths.
#'
#' Other examples:
#'
#' imeta_qu( type="d", query="a like b%")
#'
#' returns data-objects with attribute 'a' with a value that starts with 'b'.
#'
#' imeta_qu( type="d", query="a like %")
#'
#' returns data-objects with attribute 'a' defined (with any value).
#'
#' @param src_type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param query    Query string expression on AVUs
#'
imeta_qu <- function(src_type, query) {
    .Call('rirods_imeta_qu', PACKAGE = 'rirods', src_type, query)
}

#' imeta_ls
#' imeta_ls( type=[d|C|R|u], name="object_name"[, attr="AttName"])
#'
#' List defined AVUs for the specified item
#'
#' imeta_ls( type="d", name="file1")
#'
#' If the optional AttName is included, it is the attribute name
#' you wish to list and only those will be listed.
#'
#' @param type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param name iRODS object name - should be always present
#' @param attribute limit listing to values for a single attribute name defined by attr, blank for all
#'
imeta_ls <- function(type, name, attribute = "") {
    .Call('rirods_imeta_ls', PACKAGE = 'rirods', type, name, attribute)
}

#' imeta_lsw
#' imeta_lsw( type=[d|C|R|u], name="object_name"[, avu="AttName"])
#'
#' List defined AVUs for the specified item
#'
#' imeta_lsw( type="d", name="file1")
#'
#' If the optional AttName is included, it is the attribute name
#' you wish to list, doing so using wildcard matching.
#'
#' imeta_lsw(type="d", name="file1", avu="attr%")
#'
#' @param type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param name iRODS object name - should be always present
#' @param avu  format "Attribute", using wildcards '%' and '_'
#'
imeta_lsw <- function(type, name, avu) {
    .Call('rirods_imeta_lsw', PACKAGE = 'rirods', type, name, avu)
}

#' imeta
#' Retrieves the file from iRODS
#' @param command         add, ls, rm, cp, qu
#' @param object_type     iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param dst_object_type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param dst_name        iRODS object name(cp only)
#' @param avu             format "Attribute;Value;Unit"
#' @param new_avu         format "Attribute;Value;Unit"
#' @param query           See, icommand help 'imeta help qu' for details, help is printed if empty
#'
NULL

#' imeta_add
#' Adds a new metadata AVU to the selected iRODS object of the given type
#' @param type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param name iRODS object name - should be always present
#' @param avu  format "Attribute;Value;Unit"
#'
NULL

#' imeta_addw
#' Adds a new metadata AVU to the selected iRODS object of the given type
#' @param type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param name iRODS object name - should be always present, one can use wildcards %, _ on object name
#' @param avu  format "Attribute;Value;Unit"
#'
NULL

#' imeta_rm
#' Removes the metadata AVU from the selected iRODS object of the given type
#' @param type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param name iRODS object name - should be always present
#' @param avu  format "Attribute;Value;Unit", unit must be present if exists (iRODS constraint)
#'
NULL

#' imeta_rmw
#' Removes the metadata AVU from the selected iRODS object of the given type
#' @param type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param name iRODS object name - should be always present
#' @param avu  format "Attribute;Value;Unit", one can use wildcards %, _ on attribute, value, unit
#'
imeta_rmw <- function(type, name, avu) {
    .Call('rirods_imeta_rmw', PACKAGE = 'rirods', type, name, avu)
}

#' imeta_cp
#' Copies the metadata AVUs from the selected iRODS object of the given type to another object
#' @param src_type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param dst_type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param src_name iRODS source object name - should be always present
#' @param dst_name iRODS destination object name - should be always present
#'
imeta_cp <- function(src_type, dst_type, src_name, dst_name) {
    .Call('rirods_imeta_cp', PACKAGE = 'rirods', src_type, dst_type, src_name, dst_name)
}

#' imeta_mod
#' Modifies the metadata AVUs of the selected iRODS object of the given type
#' @param src_type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param src_name iRODS source object name - should be always present
#' @param old_avu  format "Attribute;Value;Unit", e.g. "Attr1;Value1;" or "Attr1;Value1;Unit1"
#' @param new_avu  format "Attribute;Value;Unit", e.g. "AttrNew;12;" or "AttrNew;12;mm"
#'
imeta_mod <- function(src_type, src_name, old_avu, new_avu) {
    .Call('rirods_imeta_mod', PACKAGE = 'rirods', src_type, src_name, old_avu, new_avu)
}

#' imeta_set
#' Set the newValue (and newUnit) of an AVU of a dataobj (d), collection(C),
#' resource(R) or user(u).
#'
#' 'imeta_set' modifies an AVU if it exists, or creates one if it does not.
#' If the AttName does not exist, or is used by multiple objects, the AVU for
#' this object is added.
#'
#' If the AttName is used only by this one object, the AVU (row) is modified
#' with the new values, reducing the database overhead (unused rows).
#'
#' imeta_set( type="d", name="file1", avu="distance;12")
#'
#' @param type iRODS files[dD], collections[cC], resources[rR], users[u]
#' @param name iRODS object name - should be always present
#' @param avu  format "Attribute;Value;Unit"
#'
imeta_set <- function(type, name, avu) {
    .Call('rirods_imeta_set', PACKAGE = 'rirods', type, name, avu)
}

#' imkdir
#' Creates a new directory in iRODS
#' @param rods_path The iRODS new directory path
#' @param parents   Create automatically all required parent directories
#'
imkdir <- function(rods_path = "", parents = FALSE) {
    .Call('rirods_imkdir', PACKAGE = 'rirods', rods_path, parents)
}

#' imv
#' imv moves/renames an irods data-object (file) or collection (directory) to
#' another, data-object or collection. The move works if both the source and
#' target are normal (registered in the iCAT) iRODS objects. It also works when
#' the source object is in a mounted collection (object not registered in the
#' iCAT) and the target is a normal object. In fact, this may provide a way
#' to design a drop box where data can be uploaded quickly into a mounted
#' collection and then in the background, moved to the eventual target
#' collection (where data are registered in the iCAT). But currently, the
#' move from a normal collection to a mounted collection is not supported.
#'
#' If you do a move and rename at the same time, for example,
#' 'imv file1 coll1/file2', it will normally succeed if there's no conflicting
#' data-object name in the source collection (file2) but fail (giving error
#' CAT_NAME_EXISTS_AS_DATAOBJ) if there is, since, internally IRODS is doing
#' a rename and then a move. Please handle this by running multiple separate
#' 'imv' commands.
#'
#' @param src_path Local source file path
#' @param dest_path iRODS destination file path
#' @param verbose verbose
#'
imv <- function(src_path, dest_path, verbose = FALSE) {
    .Call('rirods_imv', PACKAGE = 'rirods', src_path, dest_path, verbose)
}

#' ipwd
#' Returns the current working directory in iRODS or empty string if error
#'
ipwd <- function() {
    .Call('rirods_ipwd', PACKAGE = 'rirods')
}

#' imkdir
#' Creates a new directory in iRODS
#' @param rods_path The iRODS new directory path
#' @param recursive Remove all recursively
#' @param verbose   Verbose output
#' @param force     Skip trash bin, directly delete the iRODS object(s)
#'
irm <- function(rods_path = "", recursive = FALSE, verbose = FALSE, force = FALSE) {
    .Call('rirods_irm', PACKAGE = 'rirods', rods_path, recursive, verbose, force)
}

